//
//  View.m
//  Attitude
//
//  Created by NYU User on 10/13/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "View.h"


@implementation View


- (id) initWithFrame: (CGRect) frame {
	if ((self = [super initWithFrame: frame]) != nil) {
		// Initialization code
		manager = [[CMMotionManager alloc] init];
		if (!manager.isDeviceMotionAvailable) {
			//[manager release];
			[self release];
			self = nil;
			return self;
		}
		
		self.backgroundColor = [UIColor whiteColor];
		CGRect f = CGRectMake(0, 0,
			self.bounds.size.width, self.bounds.size.height);
		view = [[UITextView alloc] initWithFrame: f];
		view.backgroundColor = [UIColor clearColor];
		view.font = [UIFont systemFontOfSize: 14];
		view.editable = NO;
		[self addSubview: view];

		double hertz = 10;	//cycles per second
		manager.deviceMotionUpdateInterval = 1.0 / hertz;
		[manager startDeviceMotionUpdates];
				   
		timer = [NSTimer scheduledTimerWithTimeInterval: 1.0 / hertz
			target: self
			selector: @selector(draw)
			userInfo: nil
			repeats: YES
		];
	}
	return self;
}

#define DEGREES(radians)	((radians) * 180 / M_PI)

- (void) draw {
	CMDeviceMotion *m = manager.deviceMotion;
	CMAttitude *a = m.attitude;
	CMAcceleration gravity = m.gravity;
	
	double roll = a.roll;
	double pitch = a.pitch;
	double yaw = a.yaw;
	
	CMRotationMatrix matrix = a.rotationMatrix;
	CMQuaternion quaternion = a.quaternion;
	
	view.text = [NSString stringWithFormat:
		@"roll == %g°\n"
		"pitch == %g°\n"
		"yaw == %g°\n\n"
				 
		"%13g%13g%13g\n"
		"%13g%13g%13g\n"
		"%13g%13g%13g\n\n"
				 
		"quaternion.x == %g\n"
		"quaternion.y == %g\n"
		"quaternion.z == %g\n"
		"quaternion.w == %g\n"
		"θ == %g°\n\n"
				 
		"gravity.x == %g\n"
		"gravity.y == %g\n"
		"gravity.z == %g\n\n",
				 
		DEGREES(roll),
		DEGREES(pitch),
		DEGREES(yaw),
		
		matrix.m11, matrix.m12, matrix.m13,
		matrix.m21, matrix.m22, matrix.m23,
		matrix.m31, matrix.m32, matrix.m33,
				 
		quaternion.x,
		quaternion.y,
		quaternion.z,
		quaternion.w,
		DEGREES(2 * acos(quaternion.w)),
				 
		gravity.x,
		gravity.y,
		gravity.z
	];

	[self setNeedsDisplay];
}


// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void) drawRect: (CGRect) rect {
	// Drawing code
	//Put the origin at the center.
	CGContextRef c = UIGraphicsGetCurrentContext();
	CGSize size = self.bounds.size;
	CGContextTranslateCTM(c, size.width / 2, size.height / 2);
	
	//X axis
	CGContextBeginPath(c);
	CGContextMoveToPoint(c, -size.width / 2, 0);
	CGContextAddLineToPoint(c, size.width / 2, 0);
	
	//Y axis
	CGContextMoveToPoint(c, 0, size.height / 2);
	CGContextAddLineToPoint(c, 0, -size.height / 2);
	
	CGContextSetRGBStrokeColor(c, 0, 0, 1, 0.5);	//faint blue
	CGContextStrokePath(c);
	
	CGFloat scale = 100;
	CGContextConcatCTM(c, CGAffineTransformMakeScale(scale, -scale));
	CGContextSetLineWidth(c, 1.0 / scale);

	//Initially, the black line points to the right,
	//from (0, 0) to (1, 0).
	CMRotationMatrix m = manager.deviceMotion.attitude.rotationMatrix;
	
	CGFloat x = 1;
	CGFloat y = 0;
	CGFloat z = 0;
	
	CGFloat newX = x * m.m11 + y * m.m12 + z * m.m13;
	CGFloat newY = x * m.m21 + y * m.m22 + z * m.m23;
	//CGFloat newZ = x * m.m31 + y * m.m32 + z * m.m33;
	//NSLog(@"%g, %g, %g", newX, newY, newX);
	
	CGContextBeginPath(c);
	CGContextMoveToPoint(c, 0, 0);
	CGContextAddLineToPoint(c, newX, newY);
	CGContextSetRGBStrokeColor(c, 0, 0, 0, 1);	//black
	CGContextStrokePath(c);
}


- (void) dealloc {
	[timer invalidate];
	[view release];
	[manager release];
	[super dealloc];
}


@end
